@@ -35,10 +35,12 @@ from Products.Archetypes.public import *
 from Products.Archetypes.utils import shasattr
 from Products.Archetypes.config import LANGUAGE_DEFAULT
 
+from Products.LinguaPlone import events
 from Products.LinguaPlone import config
 from Products.LinguaPlone import permissions
 
 from zope.interface import implements
+from zope.event import notify
 from Products.LinguaPlone.interfaces import ITranslatable
 
 from Products.CMFDynamicViewFTI.interface import ISelectableBrowserDefault
@@ -122,6 +124,8 @@ class I18NBaseObject(Implicit):
         if self.hasTranslation(language):
             translation = self.getTranslation(language)
             raise AlreadyTranslated, translation.absolute_url()
+        beforeevent = events.ObjectWillBeTranslatedEvent(self, language)
+        notify(beforeevent)         
         id = canonical.getId()
         while not parent.checkIdAvailable(id):
             id = '%s-%s' % (id, language)
@@ -133,7 +137,7 @@ class I18NBaseObject(Implicit):
         # translation relationship, make sure it is done now.
         if o.getCanonical() != canonical:
             o.addTranslationReference(canonical)
-        self.invalidateTranslationCache()
+        self.invalidateTranslationCache()        
         # Copy over the language independent fields
         schema = canonical.Schema()
         independent_fields = schema.filterFields(languageIndependent=True)
@@ -142,8 +146,16 @@ class I18NBaseObject(Implicit):
             if not accessor:
                 accessor = field.getAccessor(canonical)
             data = accessor()
-            translation_mutator = getattr(o, field.translation_mutator)
-            translation_mutator(data)
+            mutatorname = getattr(field, 'translation_mutator', None)
+            if mutatorname is None:
+                # seems we have some field from archetypes.schemaextender
+                # or something else not using ClassGen
+                # fall back to default mutator
+                o.getField(field.getName()).set(self, data)
+            else:
+                # holy ClassGen crap - we have a generated method!
+                translation_mutator = getattr(o, mutatorname)
+                translation_mutator(data)
         # If this is a folder, move translated subobjects aswell.
         if self.isPrincipiaFolderish:
             moveids = []
@@ -159,6 +171,8 @@ class I18NBaseObject(Implicit):
         o.reindexObject()
         if isDefaultPage(canonical, self.REQUEST):
             o._lp_default_page = True
+        afterevent = events.ObjectTranslatedEvent(self, o, language)
+        notify(afterevent)             
 
     security.declareProtected(permissions.AddPortalContent,
                               'addTranslationReference')
@@ -187,7 +201,7 @@ class I18NBaseObject(Implicit):
                               'removeTranslationReference')
     def removeTranslationReference(self, translation):
         """Removes the translation reference."""
-        self.deleteReference(translation, config.RELATIONSHIP)
+        translation.deleteReference(self, config.RELATIONSHIP)
         self.invalidateTranslationCache()
 
     security.declareProtected(permissions.View, 'hasTranslation')
